@@ -35,6 +35,7 @@ CIM data type                             Python type
 ========================================  =====================================
 boolean                                   :class:`py:bool`
 char16                                    :term:`string`
+                                          or :class:`~pywbem.Char16`
 string                                    :term:`string`
 string (EmbeddedInstance)                 :class:`~pywbem.CIMInstance`
 string (EmbeddedObject)                   :class:`~pywbem.CIMInstance`
@@ -75,13 +76,12 @@ from __future__ import absolute_import
 
 from datetime import tzinfo, datetime, timedelta
 import re
-import warnings
 import copy
-import traceback
 import six
 
-from .config import DEBUG_WARNING_ORIGIN, ENFORCE_INTEGER_RANGE
-from ._utils import _ensure_unicode, _hash_item, _format, _to_unicode
+from .config import ENFORCE_INTEGER_RANGE
+from ._utils import _ensure_unicode, _hash_item, _format, _to_unicode, \
+    _eq_item
 
 if six.PY2:
     # pylint: disable=invalid-name,undefined-variable
@@ -94,93 +94,62 @@ else:
 __all__ = ['cimtype', 'type_from_name', 'MinutesFromUTC', 'CIMType',
            'CIMDateTime', 'CIMInt', 'Uint8', 'Sint8', 'Uint16', 'Sint16',
            'Uint32', 'Sint32', 'Uint64', 'Sint64', 'CIMFloat', 'Real32',
-           'Real64']
+           'Real64', 'Char16']
 
 
 class _CIMComparisonMixin(object):  # pylint: disable=too-few-public-methods
-    """Mixin class providing default implementations for rich comparison
-    operators.
+    """
+    Mixin class providing default implementations for equality test
+    operators and hash function.
+
+    The implementations of ordering tests are also provided and raise an
+    exception because ordering of CIM objects is not supported.
 
     In Python 2, the rich comparison operators (e.g. `__eq__()`) have
-    precedence over the traditional comparator method (`_cmp__()`).
-    In Python 3, the comparator method (`_cmp__()`) no longer exists.
+    precedence over the traditional comparator method (`__cmp__()`).
+    In Python 3, the comparator method (`__cmp__()`) no longer exists.
     Therefore, implementing the rich comparison operators works in both.
-
-    The default implementations delegate to a comparator method `_cmp()`
-    implemented by subclasses. This requires that the subclasses can
-    define total ordering. (If they cannot, this mixin class cannot be
-    used).
     """
 
     def __eq__(self, other):
         """
-        Invoked when two CIM objects are compared with the `==` operator.
+        Equality test for two CIM objects.
 
-        The comparison is delegated to the `_cmp()` method.
+        The comparison must be implemented in the derived class.
         """
-        return self._cmp(other) == 0
+        raise NotImplementedError
 
     def __ne__(self, other):
         """
-        Invoked when two CIM objects are compared with the `!=` operator.
+        Non-equality test for two CIM objects.
 
-        The comparison is delegated to the `_cmp()` method.
+        The comparison is delegated to the `__eq__()` method, because we
+        require it to be implemented.
+        See https://stackoverflow.com/a/30676267/1424462 for discussion about
+        delegating to == vs. __eq__().
         """
-        return self._cmp(other) != 0
+        return not self.__eq__(other)
 
-    def __ordering_deprecated(self):
-        """Deprecated warning for pywbem CIM Objects"""
-        msg = _format("Ordering comparisons involving {0} objects are "
-                      "deprecated.", self.__class__.__name__)
-        if DEBUG_WARNING_ORIGIN:
-            msg += "\nTraceback:\n" + ''.join(traceback.format_stack())
-        warnings.warn(msg, DeprecationWarning, stacklevel=3)
+    def __raise_ordering_not_supported(self, other, op):
+        """
+        Function to raise a TypeError indicating that ordering of this class
+        is not supported.
+        """
+        raise TypeError(
+            "'{}' not supported between instances of '{}' and '{}'".
+            format(op, type(self), type(other)))
 
     def __lt__(self, other):
-        """
-        Invoked when two CIM objects are compared with the `<` operator.
-
-        The comparison is delegated to the `_cmp()` method.
-        """
-        self.__ordering_deprecated()
-        return self._cmp(other) < 0
+        self.__raise_ordering_not_supported(other, '<')
 
     def __gt__(self, other):
-        """
-        Invoked when two CIM objects are compared with the `>` operator.
-
-        The comparison is delegated to the `_cmp()` method.
-        """
-        self.__ordering_deprecated()
-        return self._cmp(other) > 0
-
-    def __le__(self, other):
-        """
-        Invoked when two CIM objects are compared with the `<=` operator.
-
-        The comparison is delegated to the `_cmp()` method.
-        """
-        self.__ordering_deprecated()
-        return self._cmp(other) <= 0
+        self.__raise_ordering_not_supported(other, '>')
 
     def __ge__(self, other):
-        """
-        Invoked when two CIM objects are compared with the `>=` operator.
+        self.__raise_ordering_not_supported(other, '>=')
 
-        The comparison is delegated to the `_cmp()` method.
-        """
-        self.__ordering_deprecated()
-        return self._cmp(other) >= 0
-
-    def _cmp(self, other):
-        """
-        Interface definition for comparator method to be provided by
-        subclasses, as follows:
-        * If self == other, 0 must be returned.
-        * If self < other, -1 must be returned.
-        * If self > other, +1 must be returned.
-        """
-        raise NotImplementedError
+    def __le__(self, other):
+        self.__raise_ordering_not_supported(other, '<=')
 
     def __hash__(self):
         """
@@ -285,6 +254,26 @@ class MinutesFromUTC(tzinfo):
         """
         return timedelta(0)
 
+    def tzname(self, dt):  # pylint: disable=unused-argument
+        """
+        An implementation of the corresponding base class method,
+        (see :meth:`py:datetime.tzinfo.tzname` for its description),
+        which needs to return the name of the timezone of the
+        specified datetime object.
+
+        This implementation returns the timezone offset formatted as a
+        signed HH:MM string, where positive values are east of UTC.
+        """
+        # Note that divmod() and // return one less for negative numbers
+        # assuming the remainder would be added on it. For example,
+        # divmod(-5, 60) = -1, 55
+        # That is mathematically consistent, but is not what we want
+        # since we want both return values to be negative, for negative
+        # input. Therefore we handle the sign separately.
+        sign = '-' if self._offset < 0 else ''
+        hh, mm = divmod(abs(self._offset), 60)
+        return "{sign}{hh:02d}:{mm:02d}".format(sign=sign, hh=hh, mm=mm)
+
 
 class CIMType(object):  # pylint: disable=too-few-public-methods
     """Base type for all CIM data types defined in this package."""
@@ -294,7 +283,29 @@ class CIMType(object):  # pylint: disable=too-few-public-methods
     cimtype = None
 
 
-class CIMDateTime(CIMType, _CIMComparisonMixin):
+class Char16(CIMType, six.text_type):
+    """
+    A value of CIM data type char16.
+
+    This class is derived from :term:`unicode string`.
+
+    Normally, values of CIM data type char16 are represented using
+    :term:`unicode string` objects. This class can be used to represent values
+    of CIM data type char16 when it matters to distinguish them from values of
+    CIM data type string. The only situation where that matters is for
+    keybindings, because that allows properly setting the TYPE attribute on
+    KEYVALUE elements when creating the CIM-XML representation for a keybinding.
+    """
+
+    #: The name of the CIM datatype ``"char16"``
+    cimtype = 'char16'
+
+    # Changing the string content requires using __new__() instead of __init__()
+    def __new__(cls, content=''):
+        return super(Char16, cls).__new__(cls, _ensure_unicode(content))
+
+
+class CIMDateTime(_CIMComparisonMixin, CIMType):
     """
     A value of CIM data type datetime.
 
@@ -700,17 +711,13 @@ class CIMDateTime(CIMType, _CIMComparisonMixin):
     def __setstate__(self, arg):
         self.__init__(arg)
 
-    def _cmp(self, other):
-        # Defer import due to circular import dependencies:
-        from .cim_obj import cmpitem
+    def __eq__(self, other):
         if self is other:
-            return 0
-
+            return True
         if not isinstance(other, CIMDateTime):
-            return 1
-
-        return (cmpitem(self.datetime, other.datetime) or
-                cmpitem(self.timedelta, other.timedelta))
+            return False
+        return (_eq_item(self.datetime, other.datetime) and
+                _eq_item(self.timedelta, other.timedelta))
 
     def __hash__(self):
         """
@@ -1062,6 +1069,7 @@ def cimtype(obj):
         instancename_type = CIMInstanceName
     except NameError:
         # Defer import due to circular import dependencies:
+        # pylint: disable=import-outside-toplevel
         from pywbem._cim_obj import CIMInstanceName as instancename_type
     if isinstance(obj, instancename_type):
         return 'reference'
@@ -1070,6 +1078,7 @@ def cimtype(obj):
         instance_type = CIMInstance
     except NameError:
         # Defer import due to circular import dependencies:
+        # pylint: disable=import-outside-toplevel
         from pywbem._cim_obj import CIMInstance as instance_type
     if isinstance(obj, instance_type):  # embedded instance
         return 'string'
@@ -1078,6 +1087,7 @@ def cimtype(obj):
         class_type = CIMClass
     except NameError:
         # Defer import due to circular import dependencies:
+        # pylint: disable=import-outside-toplevel
         from pywbem._cim_obj import CIMClass as class_type
     if isinstance(obj, class_type):  # embedded class
         return 'string'
@@ -1150,6 +1160,7 @@ def type_from_name(type_name):
     """
     if type_name == 'reference':
         # Defer import due to circular import dependencies:
+        # pylint: disable=import-outside-toplevel
         from ._cim_obj import CIMInstanceName
         return CIMInstanceName
     try:
